from django.contrib.auth.models import User as BaseUser


class UnblindingRequestorUser(BaseUser):
    def __str__(self):
        return f"{self.first_name} {self.last_name} ({self.username})"

    class Meta:
        proxy = True
        default_permissions = ("view",)
        verbose_name = "Unblinding Requestor (User)"
        verbose_name_plural = "Unblinding Requestors (Users)"


class UnblindingReviewerUser(BaseUser):
    def __str__(self):
        return f"{self.first_name} {self.last_name} ({self.username})"

    class Meta:
        proxy = True
        default_permissions = ("view",)
        verbose_name = "Unblinding Reviewer (User)"
        verbose_name_plural = "Unblinding Reviewers (Users)"



#!/usr/bin/env python
import logging
import os
import sys
from os.path import abspath, dirname

import django
from django.conf import settings
from django.test.runner import DiscoverRunner
from edc_constants.constants import IGNORE
from edc_test_utils import DefaultTestSettings

app_name = "edc_unblinding"
base_dir = dirname(abspath(__file__))

DEFAULT_SETTINGS = DefaultTestSettings(
    calling_file=__file__,
    template_dirs=[os.path.join(base_dir, app_name, "tests", "templates")],
    BASE_DIR=base_dir,
    APP_NAME=app_name,
    ETC_DIR=os.path.join(base_dir, app_name, "tests", "etc"),
    EDC_AUTH_CODENAMES_WARN_ONLY=True,
    EDC_NAVBAR_VERIFY_ON_LOAD=IGNORE,
    EXTRA_INSTALLED_APPS=["visit_schedule_app.apps.AppConfig"],
    # INSTALLED_APPS=[
    #     "django.contrib.admin",
    #     "django.contrib.auth",
    #     "django.contrib.contenttypes",
    #     "django.contrib.sessions",
    #     "django.contrib.messages",
    #     "django.contrib.staticfiles",
    #     "django.contrib.sites",
    #     "simple_history",
    #     "django_crypto_fields.apps.AppConfig",
    #     "edc_appointment.apps.AppConfig",
    #     "edc_crf.apps.AppConfig",
    #     "edc_action_item.apps.AppConfig",
    #     "edc_adverse_event.apps.AppConfig",
    #     "adverse_event_app.apps.AppConfig",
    #     "edc_dashboard.apps.AppConfig",
    #     "edc_consent.apps.AppConfig",
    #     "edc_facility.apps.AppConfig",
    #     "edc_metadata.apps.AppConfig",
    #     "edc_notification.apps.AppConfig",
    #     "edc_device.apps.AppConfig",
    #     "edc_identifier.apps.AppConfig",
    #     "edc_registration.apps.AppConfig",
    #     "edc_sites.apps.AppConfig",
    #     "edc_timepoint.apps.AppConfig",
    #     "edc_visit_schedule.apps.AppConfig",
    #     "visit_schedule_app.apps.AppConfig",
    #     "edc_unblinding.apps.AppConfig",
    #     "edc_auth.apps.AppConfig",
    # ],
    DASHBOARD_BASE_TEMPLATES={
        "dashboard_template": os.path.join(
            base_dir, "edc_unblinding", "tests", "templates", "dashboard.html"
        ),
        "dashboard2_template": os.path.join(
            base_dir, "edc_unblinding", "tests", "templates", "dashboard2.html"
        ),
    },
    EDC_SITES_DEFAULT_COUNTRY="tanzania",
    use_test_urls=True,
    add_dashboard_middleware=True,
    add_lab_dashboard_middleware=True,
).settings


def main():
    if not settings.configured:
        settings.configure(**DEFAULT_SETTINGS)
    django.setup()
    tags = [t.split("=")[1] for t in sys.argv if t.startswith("--tag")]
    failures = DiscoverRunner(failfast=False, tags=tags).run_tests([f"{app_name}.tests"])
    sys.exit(failures)


if __name__ == "__main__":
    logging.basicConfig()
    main()



from edc_auth.site_auths import site_auths

from .auth_objects import (
    UNBLINDING_REQUESTORS,
    UNBLINDING_REQUESTORS_ROLE,
    UNBLINDING_REVIEWERS,
    UNBLINDING_REVIEWERS_ROLE,
    unblinding_requestors,
    unblinding_reviewers,
)

site_auths.add_group(*unblinding_requestors, name=UNBLINDING_REQUESTORS)
site_auths.add_group(*unblinding_reviewers, name=UNBLINDING_REVIEWERS)
site_auths.add_role(UNBLINDING_REQUESTORS, name=UNBLINDING_REQUESTORS_ROLE)
site_auths.add_role(UNBLINDING_REVIEWERS, name=UNBLINDING_REVIEWERS_ROLE)



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import action_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import edc_unblinding_admin
from ..forms import UnblindingRequestForm
from ..models import UnblindingRequest


@admin.register(UnblindingRequest, site=edc_unblinding_admin)
class UnblindingRequestAdmin(ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):

    form = UnblindingRequestForm

    additional_instructions = (
        "Note: if the patient is deceased, complete the Death Report "
        "before completing this form. "
    )

    fieldsets = (
        (
            "Request",
            {
                "fields": (
                    "report_datetime",
                    "subject_identifier",
                    "initials",
                    "requestor",
                    "unblinding_reason",
                )
            },
        ),
        ("Approval", {"fields": ("approved", "approved_datetime")}),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    autocomplete_fields = ["requestor"]

    readonly_fields = ("approved", "approved_datetime")

    radio_fields = {"approved": admin.VERTICAL}

    list_display = (
        "subject_identifier",
        "dashboard",
        "requestor",
        "approved",
        "approved_datetime",
        "action_identifier",
        "created",
    )



# Generated by Django 3.2.6 on 2021-09-08 20:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("edc_unblinding", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalunblindingrequest",
            name="requestor",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Choose a name from the list",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_unblinding.unblindingrequestoruser",
                verbose_name="Unblinding requested by",
            ),
        ),
        migrations.AlterField(
            model_name="historicalunblindingreview",
            name="reviewer",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Choose a name from the list",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_unblinding.unblindingrevieweruser",
                verbose_name="Unblinding request reviewed by",
            ),
        ),
        migrations.AlterField(
            model_name="unblindingrequest",
            name="requestor",
            field=models.ForeignKey(
                help_text="Choose a name from the list",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_unblinding.unblindingrequestoruser",
                verbose_name="Unblinding requested by",
            ),
        ),
        migrations.AlterField(
            model_name="unblindingreview",
            name="reviewer",
            field=models.ForeignKey(
                help_text="Choose a name from the list",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_unblinding.unblindingrevieweruser",
                verbose_name="Unblinding request reviewed by",
            ),
        ),
    ]



from django.db import models
from edc_action_item.models.action_model_mixin import ActionModelMixin
from edc_constants.choices import YES_NO_TBD
from edc_constants.constants import TBD
from edc_identifier.managers import SubjectIdentifierManager
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_model.models.base_uuid_model import BaseUuidModel
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils.date import get_utcnow

from ..constants import UNBLINDING_REVIEW_ACTION
from .unblinding_user import UnblindingReviewerUser


class UnblindingReview(
    NonUniqueSubjectIdentifierFieldMixin,
    SiteModelMixin,
    ActionModelMixin,
    BaseUuidModel,
):

    action_name = UNBLINDING_REVIEW_ACTION

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=get_utcnow
    )

    reviewer = models.ForeignKey(
        UnblindingReviewerUser,
        related_name="+",
        on_delete=models.PROTECT,
        verbose_name="Unblinding request reviewed by",
        help_text="Choose a name from the list",
    )

    approved = models.CharField(max_length=15, default=TBD, choices=YES_NO_TBD)

    comment = models.TextField(verbose_name="Comment", null=True)

    on_site = CurrentSiteManager()

    objects = SubjectIdentifierManager()

    def natural_key(self):
        return (self.action_identifier,)

    class Meta:
        verbose_name = "Unblinding Review"
        verbose_name_plural = "Unblinding Reviews"
        indexes = [
            models.Index(fields=["subject_identifier", "action_identifier", "site", "id"])
        ]



from django.contrib.admin.decorators import register
from django.contrib.sites.shortcuts import get_current_site
from edc_auth.admin import UserAdmin as BaseUserAdmin

from ..admin_site import edc_unblinding_admin
from ..auth_objects import UNBLINDING_REQUESTORS, UNBLINDING_REVIEWERS
from ..models import UnblindingRequestorUser, UnblindingReviewerUser


@register(UnblindingRequestorUser, site=edc_unblinding_admin)
class UnblindingRequestorUserAdmin(BaseUserAdmin):
    ordering = ("first_name", "last_name")
    search_fields = ("first_name", "last_name", "username", "email")

    inlines = []

    populate_data_dictionary = False

    def get_queryset(self, request):
        opts = dict(
            userprofile__sites__id=get_current_site(request).id,
            groups__name__in=[UNBLINDING_REQUESTORS],
        )
        return super().get_queryset(request).filter(**opts)


@register(UnblindingReviewerUser, site=edc_unblinding_admin)
class UnblindingReviewerUserAdmin(BaseUserAdmin):
    ordering = ("first_name", "last_name")
    search_fields = ("first_name", "last_name", "username", "email")

    inlines = []

    populate_data_dictionary = False

    def get_queryset(self, request):
        opts = dict(
            userprofile__sites__id=get_current_site(request).id,
            groups__name__in=[UNBLINDING_REVIEWERS],
        )
        return super().get_queryset(request).filter(**opts)



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import action_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import edc_unblinding_admin
from ..forms import UnblindingReviewForm
from ..models import UnblindingReview


@admin.register(UnblindingReview, site=edc_unblinding_admin)
class UnblindingReviewAdmin(ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):

    form = UnblindingReviewForm

    fieldsets = (
        ("Request", {"fields": ("subject_identifier", "report_datetime", "reviewer")}),
        ("Approval", {"fields": ("approved", "comment")}),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    autocomplete_fields = ["reviewer"]

    list_display = (
        "subject_identifier",
        "dashboard",
        "report_datetime",
        "reviewer",
        "approved",
        "action_identifier",
        "created",
    )

    radio_fields = {"approved": admin.VERTICAL}



from django.contrib.auth import get_user_model
from django.test import TestCase, override_settings
from edc_action_item import site_action_items
from edc_auth.site_auths import site_auths
from edc_consent import site_consents
from edc_facility import import_holidays
from edc_reference import site_reference_configs
from edc_visit_schedule import site_visit_schedules
from edc_visit_tracking.tests.helper import Helper
from visit_schedule_app.consents import v1_consent
from visit_schedule_app.models import SubjectConsent
from visit_schedule_app.visit_schedule import visit_schedule

from edc_unblinding.action_items import UnblindingRequestAction, UnblindingReviewAction
from edc_unblinding.auth_objects import (
    UNBLINDING_REQUESTORS_ROLE,
    UNBLINDING_REVIEWERS_ROLE,
)
from edc_unblinding.models import UnblindingRequest, UnblindingRequestorUser


@override_settings(
    SUBJECT_CONSENT_MODEL="visit_schedule_app.subjectconsent",
    SUBJECT_SCREENING_MODEL="visit_schedule_app.subjectscreening",
)
class EdcunblindingTestCase(TestCase):
    helper_cls = Helper

    @classmethod
    def setUpClass(cls):
        site_action_items.register(action_cls=UnblindingRequestAction)
        site_action_items.register(action_cls=UnblindingReviewAction)
        return super().setUpClass()

    @classmethod
    def setUpTestData(cls):
        import_holidays()
        get_user_model().objects.create(username="frazey", is_staff=True, is_active=True)

    def setUp(self):
        self.user = get_user_model().objects.get(username="frazey")
        self.subject_identifier = "12345"
        site_consents.registry = {}
        site_consents.register(v1_consent)
        self.helper = self.helper_cls(
            subject_identifier=self.subject_identifier,
            subject_consent_model_cls=SubjectConsent,
            onschedule_model_name="visit_schedule_app.onschedule",
        )
        site_visit_schedules._registry = {}
        site_visit_schedules.register(visit_schedule=visit_schedule)
        site_reference_configs.register_from_visit_schedule(
            visit_models={"edc_appointment.appointment": "visit_schedule_app.subjectvisit"}
        )
        self.subject_consent = self.helper.consent_and_put_on_schedule()

    def test_ok(self):
        opts = dict(
            subject_identifier=self.subject_consent.subject_identifier,
            requestor=UnblindingRequestorUser.objects.all()[0],
        )
        model = UnblindingRequest(**opts)
        model.save()

    def test_auth(self):
        self.assertIn(UNBLINDING_REQUESTORS_ROLE, site_auths.roles)
        self.assertIn(UNBLINDING_REVIEWERS_ROLE, site_auths.roles)
